"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const child_process_1 = require("child_process");
const enquirer = require("enquirer");
const fs = require("fs");
const path = require("path");
const sinon = require("sinon");
const util = require("util");
const conf_1 = require("../../src/services/conf");
const docker_1 = require("../../src/services/docker");
const util_1 = require("../../src/util");
const originCwd = process.cwd();
const { resolve } = path;
const asyncExec = util.promisify(child_process_1.exec);
const sandbox = sinon.createSandbox();
const { RdTypes } = conf_1.default;
const framework = 'vue';
const project = 'demo-app';
const cmdDir = resolve(__dirname, '../../src/commands');
const projectDir = resolve(__dirname, '../.tmp/', project);
let CmdCreate = null;
let CmdLint = null;
let CmdRun = null;
let rdType = RdTypes.Application;
const PromptStub = question => {
    if (question instanceof Array) {
        return {
            type: rdType,
            framework,
        };
    }
    if (question.name === 'name') {
        return {
            name: project
        };
    }
    if (question.name === 'rdcName') {
        return {
            rdcName: conf_1.default.frameworks[framework].rdcStarter
        };
    }
    if (question.name === 'version') {
        return {
            version: 'latest'
        };
    }
    if (question.name === 'rdcRepo') {
        return {
            rdcRepo: 'rdebase/rdc-vue-starter'
        };
    }
};
describe('app', async () => {
    before(async () => {
        sandbox.stub(enquirer, 'prompt').get(() => PromptStub);
        CmdCreate = util_1.default.ensureRequire(resolve(cmdDir, 'create.ts')).default;
        await asyncExec('mkdir -p ./test/.tmp');
        process.chdir('test/.tmp');
        await CmdCreate.run([]);
    });
    after(async () => {
        sandbox.restore();
        process.chdir(originCwd);
        await asyncExec(`rm -rf ./test/.tmp/${project}`);
    });
    describe('create', async () => {
        it('should create a dir called app', async () => {
            const appPath = resolve(projectDir, 'app');
            chai_1.expect(fs.existsSync(appPath)).to.be.true;
        });
        it('should create a file called rda.config.js', async () => {
            const confPath = resolve(projectDir, 'rda.config.js');
            chai_1.expect(fs.existsSync(confPath)).to.be.true;
        });
        it('should create an readme.md file', async () => {
            const readMe = resolve(projectDir, 'README.md');
            chai_1.expect(fs.existsSync(readMe)).to.be.true;
        });
        it('should gen .gitignore file', async () => {
            const gitIgnoreFile = resolve(projectDir, '.gitignore');
            chai_1.expect(fs.existsSync(gitIgnoreFile)).to.be.true;
        });
        it('should gen pre-commit file in .git dir', async () => {
            const precommit = resolve(projectDir, '.git/hooks/pre-commit');
            chai_1.expect(fs.existsSync(precommit)).to.be.true;
        });
        it('should run rde lint without error', async () => {
            const fake = sinon.fake();
            try {
                process.chdir(projectDir);
                CmdLint = util_1.default.ensureRequire(resolve(cmdDir, 'lint.ts')).default;
                await CmdLint.run([]);
                fake();
            }
            catch (e) {
                chai_1.expect(e).is.not.null;
                chai_1.expect(fake.callCount).equal(1);
            }
            finally {
                process.chdir('../');
            }
        });
        describe('rda.config.js', async () => {
            let confPath = null;
            let rdaConf = null;
            let container = null;
            let docker = null;
            let suites = null;
            before(async () => {
                confPath = resolve(projectDir, 'rda.config.js');
                rdaConf = util_1.default.ensureRequire(confPath);
                container = rdaConf.container;
                docker = rdaConf.docker;
                suites = rdaConf.suites;
            });
            it('should contain container prop with name and docs value', async () => {
                const defaultRdc = conf_1.default.frameworks[framework].rdcStarter;
                chai_1.expect(container.name).to.equal(`${defaultRdc}:latest`);
                chai_1.expect(container.docs.length).is.greaterThan(0);
            });
            it('should contain docker prop with ports value', async () => {
                chai_1.expect(docker.ports.length).is.greaterThan(0);
            });
            it('should contain an empty suites prop', async () => {
                chai_1.expect(suites.length).to.equal(0);
            });
        });
    });
    describe('run', async () => {
        const appImg = `dev-${project}`;
        before(async () => {
            process.chdir(projectDir);
            CmdRun = util_1.default.ensureRequire(resolve(cmdDir, 'run.ts')).default;
            // remove created images previously
            if (await docker_1.default.imageExist(appImg)) {
                // it will not await till the cmd end
                // await asyncExec(`docker rmi $(docker images ${appImg} -q)`)
            }
        });
        it('should throw error if no config file found', async () => {
            const fake = sinon.fake();
            await asyncExec('mv rda.config.js rda.config.js_backup');
            try {
                await CmdRun.run(['lint']);
                fake();
            }
            catch (e) {
                if (e) { }
            }
            finally {
                chai_1.expect(fake.callCount).to.equal(0);
                await asyncExec('mv rda.config.js_backup rda.config.js');
            }
        });
        it('should create .cache dir with Dockerfile, docker-compose.yml, package.json, .cache file', async () => {
            const cachePath = resolve(projectDir, '.cache');
            const dockerfilePath = resolve(cachePath, 'Dockerfile');
            const dockerComposePath = resolve(cachePath, 'docker-compose.yml');
            const pkgJsonPath = resolve(cachePath, 'package.json');
            const cacheFilePath = resolve(cachePath, '.cache');
            chai_1.expect(fs.existsSync(cachePath)).to.be.true;
            chai_1.expect(fs.existsSync(dockerfilePath)).to.be.true;
            chai_1.expect(fs.existsSync(dockerComposePath)).to.be.true;
            chai_1.expect(fs.existsSync(pkgJsonPath)).to.be.true;
            chai_1.expect(fs.existsSync(cacheFilePath)).to.be.true;
        });
        it('should create an image called dev-appname', async () => {
            const exist = await docker_1.default.imageExist(appImg);
            chai_1.expect(exist).to.be.true;
        });
        it('should create node_modules dir at docker path /usr/rde', async () => {
            const fake = sinon.fake();
            try {
                await asyncExec(`docker run --rm ${appImg} ls /usr/rde/node_modules`);
                fake();
            }
            catch (e) {
                if (e) { }
            }
            finally {
                chai_1.expect(fake.callCount).to.equal(1);
            }
        });
        it('should mount app dir and rda.config.js file to rdc dir inside docker', async () => {
            const composePath = resolve(projectDir, '.cache', 'docker-compose.yml');
            const text = fs.readFileSync(composePath, 'utf-8');
            const regexp = /\.\.\/app:\/usr\/rde\/rdebase\/rdc-vue-starter\/app/gm;
            chai_1.expect(regexp.test(text)).to.be.true;
        });
        it('should create .rde dir in docker container', async () => {
            // docker commit a new image from running container, then inspect
        });
        describe('rebuild', async () => {
            it('should rebuild when local image not build', async () => { });
            it('should rebuild when local package.json in cache has changed', async () => { });
            it('should rebuild when rdc version changed if app', async () => { });
            it('should rebuild when rdc extends has changed if container', async () => { });
        });
    });
});
