"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const child_process_1 = require("child_process");
const enquirer = require("enquirer");
const fs = require("fs");
const path = require("path");
const sinon = require("sinon");
const util = require("util");
const docker_run_1 = require("../../src/core/docker.run");
const conf_1 = require("../../src/services/conf");
const docker_1 = require("../../src/services/docker");
const watcher_1 = require("../../src/services/watcher");
const util_1 = require("../../src/util");
const originCwd = process.cwd();
const { resolve } = path;
const asyncExec = util.promisify(child_process_1.exec);
const sandbox = sinon.createSandbox();
const { RdTypes } = conf_1.default;
const framework = 'vue';
const project = 'demo-container';
const cmdDir = resolve(__dirname, '../../src/commands');
const projectDir = resolve(__dirname, '../.tmp/', project);
let CmdCreate = null;
let CmdLint = null;
let CmdRun = null;
let CmdDockerRun = null;
let rdType = RdTypes.Container;
const PromptStub = question => {
    if (question instanceof Array) {
        return {
            type: rdType,
            framework,
        };
    }
    if (question.name === 'name') {
        return {
            name: project
        };
    }
    if (question.name === 'rdcName') {
        return {
            rdcName: conf_1.default.frameworks[framework].rdcStarter
        };
    }
    if (question.name === 'version') {
        return {
            version: 'latest'
        };
    }
    if (question.name === 'rdcRepo') {
        return {
            rdcRepo: 'rdebase/rdc-vue-starter'
        };
    }
};
describe('container', async () => {
    before(async () => {
        sandbox.stub(enquirer, 'prompt').get(() => PromptStub);
        CmdCreate = util_1.default.ensureRequire(resolve(cmdDir, 'create.ts')).default;
        await asyncExec('mkdir -p ./test/.tmp');
        process.chdir('test/.tmp');
        await CmdCreate.run([]);
    });
    after(async () => {
        sandbox.restore();
        process.chdir(originCwd);
        await asyncExec(`rm -rf ./test/.tmp/${project}`);
    });
    describe('create', async () => {
        it('should create app/template dir, and a rdc.config.js file', async () => {
            const appPath = resolve(projectDir, 'app');
            const tplPath = resolve(projectDir, 'template');
            const confPath = resolve(projectDir, 'rdc.config.js');
            chai_1.expect(fs.existsSync(appPath)).to.be.true;
            chai_1.expect(fs.existsSync(tplPath)).to.be.true;
            chai_1.expect(fs.existsSync(confPath)).to.be.true;
        });
        it('should create an readme.md file', async () => {
            const readme = resolve(projectDir, 'README.md');
            chai_1.expect(fs.existsSync(readme)).to.be.true;
        });
        it('should gen .gitignore file', async () => {
            const gitIgnoreFile = resolve(projectDir, '.gitignore');
            chai_1.expect(fs.existsSync(gitIgnoreFile)).to.be.true;
        });
        it('should gen pre-commit file in .git dir', async () => {
            const precommit = resolve(projectDir, '.git/hooks/pre-commit');
            chai_1.expect(fs.existsSync(precommit)).to.be.true;
        });
        it('should run rde lint without error', async () => {
            const fake = sinon.fake();
            try {
                process.chdir(projectDir);
                CmdLint = util_1.default.ensureRequire(resolve(cmdDir, 'lint.ts')).default;
                await CmdLint.run([]);
                fake();
            }
            catch (e) {
                chai_1.expect(e).is.not.null;
            }
            finally {
                chai_1.expect(fake.callCount).equal(1);
                process.chdir('../');
            }
        });
        describe('should create a file called rdc.config.js', async () => {
            let rdcConf = null;
            let framework = '';
            let docker = null;
            let mappings = null;
            before(async () => {
                rdcConf = util_1.default.ensureRequire(resolve(projectDir, 'rdc.config.js'));
                framework = rdcConf.framework;
                docker = rdcConf.docker;
                mappings = rdcConf.mappings;
            });
            it('should contain framework prop', async () => {
                chai_1.expect(framework).to.equal('vue');
            });
            it('should contain docker prop with tag', async () => {
                chai_1.expect(docker.tag).to.exist;
            });
            it('should contain docker prop with ports', async () => {
                chai_1.expect(docker.ports).to.exist;
            });
            it('should contain a empty mapping prop', async () => {
                chai_1.expect(mappings.length).to.greaterThan(0);
            });
        });
    });
    describe('run', async () => {
        const rdcImg = `dev-${conf_1.default.frameworks[framework].rdcStarter}`;
        before(async () => {
            process.chdir(projectDir);
            CmdRun = util_1.default.ensureRequire(resolve(cmdDir, 'run.ts')).default;
            // remove created images previously
            if (await docker_1.default.imageExist(rdcImg)) {
                // it will not await till the cmd end
                // await asyncExec(`docker rmi $(docker images ${appImg} -q)`)
            }
        });
        it('should throw error if no config file found', async () => {
            const fake = sinon.fake();
            await asyncExec('mv rdc.config.js rdc.config.js_backup');
            try {
                await CmdRun.run(['lint']);
                fake();
            }
            catch (e) {
                if (e) { }
            }
            finally {
                chai_1.expect(fake.callCount).to.equal(0);
                await asyncExec('mv rdc.config.js_backup rdc.config.js');
            }
        });
        it('should create .cache dir with Dockerfile, docker-compose.yml, package.json, .cache file', async () => {
            const cachePath = resolve(projectDir, '.cache');
            const dockerfilePath = resolve(cachePath, 'Dockerfile');
            const dockerComposePath = resolve(cachePath, 'docker-compose.yml');
            const pkgJsonPath = resolve(cachePath, 'package.json');
            const cacheFilePath = resolve(cachePath, '.cache');
            chai_1.expect(fs.existsSync(cachePath)).to.be.true;
            chai_1.expect(fs.existsSync(dockerfilePath)).to.be.true;
            chai_1.expect(fs.existsSync(dockerComposePath)).to.be.true;
            chai_1.expect(fs.existsSync(pkgJsonPath)).to.be.true;
            chai_1.expect(fs.existsSync(cacheFilePath)).to.be.true;
        });
        it('should create an image called dev-rdcname', async () => {
            const exist = await docker_1.default.imageExist(rdcImg);
            chai_1.expect(exist).to.be.true;
        });
        it('should create node_modules dir at docker path /usr/rde', async () => {
            const fake = sinon.fake();
            try {
                await asyncExec(`docker run --rm ${rdcImg} ls /usr/rde/node_modules`);
                fake();
            }
            catch (e) {
                if (e) { }
            }
            finally {
                chai_1.expect(fake.callCount).to.equal(1);
            }
        });
        it('should mount template dir and rdc.config.js file to rdc dir inside docker', async () => {
            const composePath = resolve(projectDir, '.cache', 'docker-compose.yml');
            const text = fs.readFileSync(composePath, 'utf-8');
            const regexp = /\.\.\/template:\/usr\/rde\/rdebase\/rdc-vue-starter\/template/gm;
            chai_1.expect(regexp.test(text)).to.be.true;
        });
        it('should create .rde dir in docker container', async () => {
            // docker commit a new image from running container, then inspect
        });
        describe('rebuild', async () => {
            it('should rebuild when local image not build', async () => { });
            it('should rebuild when local package.json in cache has changed', async () => { });
            it('should rebuild when rdc version changed if app', async () => { });
            it('should rebuild when rdc extends has changed if container', async () => { });
        });
    });
    describe('docker:run', async () => {
        before(async () => {
            process.chdir(projectDir);
            CmdDockerRun = util_1.default.ensureRequire(resolve(cmdDir, 'docker/run.ts')).default;
            await CmdDockerRun.run([]);
        });
        after(async () => {
            sandbox.restore();
        });
        it('should not regen .rde dir if .rde dir exist and rdtype is application', async () => {
            await asyncExec('echo 123 > rda.config.js');
            const stub = sandbox.stub(fs, 'existsSync');
            const appDir = resolve(`${conf_1.default.cwd}/app`);
            const spy = sandbox.spy(docker_run_1.default.prototype, 'composeRdcChain');
            stub.withArgs(conf_1.default.appConfPath).returns(true);
            stub.withArgs(conf_1.default.rdcConfPath).returns(false);
            stub.withArgs(appDir).returns(true);
            stub.withArgs(conf_1.default.runtimeDir).returns(true);
            await CmdDockerRun.run([]);
            chai_1.expect(spy.callCount).to.equal(0);
            await asyncExec('rm rda.config.js');
            sandbox.restore();
        });
        it('should call watcher start if run with -w flag', async () => {
            let watchCalled = false;
            sandbox.stub(watcher_1.default.prototype, 'start').callsFake(() => {
                watchCalled = true;
            });
            await CmdDockerRun.run(['-w']);
            chai_1.expect(watchCalled).to.be.true;
            sandbox.restore();
        });
    });
});
